<?php
/*-----------------------------------*
* Proyecto   : HAWAI                *
* Autor      : Federico A. Ocampo   *
* Desc: Interfaz web para el Anamov *
*-----------------------------------*/
//-----------------------------
require_once "AppConfig.php";
//-----------------------------

/*****************************************************************************
.::. Clase GestorReporte .::.
@Autor:   Federico A. Ocampo
@Email:   federico@mcanet.com.ar
@Empresa: MCAnet
@Descripcion: Clase que gestiona el llamado al generador de reportes, el parseo
				del reporte XML recibido y el formateo de este en una tabla HTML
*****************************************************************************/

class ReporteToTabla
{
	//----------Propiedades Privadas--------
	var $reportParser;  //parser SAX del XML
	var $appConfig;     //Clase de configracion de la aplicacion
	var $archivoXML;    //Path de ubicacion del archivo XML a parsear
	//...para el parseo del XML
	var $numColumnas;   //Numero de columnas que tiene el ENCOLUMNADO
	var $tablaFinal;    //Tabla Resultado del reporte
	var $seccionActual; //Indica si el nodo actual se encuentra en la cabecera o dentro de grupos
	var $posValorCorte; //Indice para ubicar los valores de los CORTEs en la columna correcta
	var $numFilaActual; //Indica el numero de fila mostrada, indica si es par o impar para css
	var $numColActual;  //Indica el numero de columna mostrada, para no imprimir columnas de mas
	//--------------------------------------


	//===================================INTERFAZ PUBLICA=============================================/

	/**
	* Constructor de la clase. Inicializa las propiedades <var>appConfig</var> y <var>archivoXML</var>
	* @param AppConfig Objeto con la configuracion de la aplicacion
	* @param string <var>archivoXML</var> path completo de ubicacion del archivo XMl a formatear
	*/
	function ReporteToTabla(&$aplicationConfig, $reporteXML)
	{
		$this->appConfig  =& $aplicationConfig;
		$this->archivoXML = $reporteXML;
		//crea el parser SAX para el archivo
		$this->reportParser = xml_parser_create();
		$this->tablaFinal    = "";
		$this->numColumnas   = 0;
		$this->seccionActual = "";
		$this->posValorCorte = 0;
		$this->numFilaActual = 0;
		$this->numColActual  = 0;
			
		
		//Configura las funciones a utilizar, por el parser SAX, para cada elemento del XML
		echo "Creando objeto reporte<br />";
		xml_set_object($this->reportParser, $this);
		xml_set_element_handler($this->reportParser, "manejarTagApertura", "manejarTagCierre");
		xml_set_character_data_handler($this->reportParser, "manejarTagTexto");
		xml_parser_set_option($this->reportParser, XML_OPTION_CASE_FOLDING, false);
	}


	/**
	* Esta funcion usa el parser SAX de PHP para parsear el archivo recibido y armar el HTML de la tabla
	* @return string Retorna una tabla HTML con los datos del XML. Si hay algun error retorna <var>null</var>
	*/
	function crearTabla()
	{
		
		//para control de errores y avisos
		$NIVEL_DEBUG = $this->appConfig->DEBUG["nivel"];
	
		// Retorno de la funcion
		$ret_tabla = "";
	
		//Abre el archivo XML chequeando que la operacion sea correcta
		$fileHandler = fopen($this->archivoXML,'r');
		trigger_error("Paso por aqui: ".$this->archivoXML, E_USER_NOTICE);
		
		
		if(!$fileHandler)
		{
			trigger_error("Error al abrir el XML {$this->archivoXML}", E_USER_ERROR);
			trigger_error("Error al abrir el XML: ".$this->archivoXML, E_USER_NOTICE);
		}
		else
		{
			/* ************VALIDAR CON XSD***************** */
			$datosArchivo = "";
			//Recorre todo el archivo de a 4kb llamando a las funciones correspondientes en cada TAG
			$v_i=0;
			while($datosArchivo = fread($fileHandler,4096))
			{
				//FILATRADO DEL ARCHIVO XML
				
				//AMPERSANDS
				$datosArchivo=str_replace("&", '&amp;', $datosArchivo);
				
				//ESC+LARGO DEL REPORTE (flenght)
				$lineacab=chr(27).'[66l';
				$datosArchivo=str_replace($lineacab, '<br>', $datosArchivo);
				
				//Ctrl+L
				$ctrl=chr(12).'L';
				$datosArchivo=str_replace(chr(12), '', $datosArchivo);
				
				//trigger_error("Datos del archivo: ". $datosArchivo, E_USER_NOTICE);
		
				$parseoCorrecto = xml_parse($this->reportParser, $datosArchivo, feof($fileHandler));
		
				//si existen errores en el parseo...
				if(!$parseoCorrecto)
				{
					$err = sprintf("XML error: %s at line %d",
								xml_error_string(xml_get_error_code($this->reportParser)),
								xml_get_current_line_number($this->reportParser));
					trigger_error($err, E_USER_NOTICE);
//					trigger_error($err, E_USER_ERROR);
					
				}
				$v_i++;
			}
			trigger_error("Hizo ".$v_i." bucles", E_USER_NOTICE);
			
			//libera el parser
			xml_parser_free($this->reportParser);
	
			//La global tablaFinal guarda el codigo HTML de la tabla
			$ret_tabla = $this->tablaFinal;
		}

		return $ret_tabla;
	}


	/**
	* Recibe un XML con mensaje de error y advertencias y lo formatea en HTML mediante un parser DOM
	* @return string Tabla HTML con los mensajes de error del XML. Si hay algun error retorna <var>null</var>
	*/
	function formatearXmlErrores()
	{

	//Incluye el archivo del parser DOM (Clase DOMIT)
	//Lo coloco aca y no al comienzo del archivo por que se llama solo cuando hay errores y asi evito
	//cargar todos los modulos del DOMIT innecesariamente cuando el reporte es valido (caso comun)
	require_once "xml/xml_domit_lite_include.php";

	/** Retorno de la funcion */
	$ret_tabla = "";
	/** Crea la clase para parsear el XML */
	$domMensajes =& new DOMIT_Lite_Document();
	$parseoCorrecto = $domMensajes->loadXML($this->archivoXML);

	if($parseoCorrecto)
	{
		/** Arma el encabezado de la tabla, con el elemento root del XML como CAPTION */
		$ret_tabla = "<table>\n" .
					"<thead>". $domMensajes->documentElement->nodeName ."</thead>\n";
		/** aliasing al arreglo con los nodos hijo, si es que tiene (nodos Cabecera) */
		$nodosHijos = null;
		if($domMensajes->documentElement->hasChildNodes()) {
		$nodosHijos =& $domMensajes->documentElement->childNodes;
		}
		/** aliasing a la cantidad de nodos hijos */
		$numNodosHijos =& $domMensajes->documentElement->childCount;

		/**
		Recorre todos los nodos hijos (cabecera) armando la tabla correspondiente
		<tr><td>titulo_n</td><td>contenido_n</td></tr>
		*/
		$ret_tabla .= "<tr>\n\t<td><b>Titulo</b></td><td><b>Contenido</b></td>\n";
		for($nodo=0; $nodo<$numNodosHijos; $nodo++)
		{
		/** Por cada uno de los nodos hijo, lee el valor que contiene */
		$msgAux = trim($nodosHijos[$nodo]->childNodes["mensaje"]->firstChild->nodeValue);
		$ret_tabla .= "<tr>\n\t<td>$msgAux</td>\n</tr>\n";
		}
		$ret_tabla .= "</table>\n";
	}
	else
	{
		$errMsg = "Archivo XML incorrecto\n Error number " . $domMensajes->getErrorCode() .": ".
				$domMensajes->getErrorString();
		trigger_error($errMsg, E_USER_ERROR);
	}

	return $ret_tabla;
	}


	//===================================INTERFAZ PRIVADA=============================================/

	/**
	* Funcion llamada por el parser SAX cada vez que se encuentra un tag de apertura, segun el nombre del
	* tag y sus atributos arma el codigo HTML de la tabla
	* @param Parser Parser SAX con la estructura del archivo
	* @param string <var>nombre</var> nombre del nodo actual
	* @param string <var>atributos</var> atributos del nodo actual
	*/
	function manejarTagApertura($parser, $nombre, $atributos)
	{

	switch($nombre)
	{
		case "Reporte":
		$this->tablaFinal .= "<table class=\"reporte\">\n";
		break;
		case "Cabecera":
		$this->seccionActual = $nombre;
		$this->tablaFinal .= "<tr>\n\t<td>\n<table class=\"cabeceraReporte\">\n<tr>\n".
						"\t<td><h2>ANAMOV-1M</h2></td>\n".
						"\t<td>{$atributos["Titulo"]} </td>\n" .
						"\t<td>{$atributos["Empresa"]}</td>\n".
						"\t<td>{$atributos["Fecha"]} - {$atributos["Hora"]}</td>\n</tr>\n";
		break;
		case "Seleccion":
		$this->tablaFinal .= "<tr>\n\t<td>{$atributos["DescSeleccion"]}</td>\n\t<td colspan=\"3\">";
		break;
		case "ValorSeleccion":
		$this->tablaFinal .= "{$atributos["DescCondicion"]} {$atributos["ValorCondicion"]}";
		break;
		case "ValorEstado":
		$this->tablaFinal .= "/{$atributos["DescCondicion"]} {$atributos["ValorCondicion"]}";
		break;
		case "Encolumnado":
		$this->seccionActual = $nombre;
		$this->tablaFinal .= "<tr>\n\t<td>\n<table class=\"datosReporte\">\n<tr class=\"nombresColumnas\">\n";
		break;
		case "Columna":
		if($this->seccionActual == "CorteGrupo" || $this->seccionActual == "CorteReporte" ||
			$this->seccionActual == "InicioGrupo")
		{
			$this->posValorCorte = $atributos["Id"];
			$this->tablaFinal .= "<tr class=\"filaCorte\">\n";
		}
		else
		{

			if($this->seccionActual == "Encolumnado")
			{
			$this->tablaFinal .= "\t<td ";
			$this->numColumnas++;
			$ancho_nombre = strlen($atributos["DescColumna"]);
			$ancho_mask = strlen($atributos["Mascara"] );
			if($ancho_nombre < $ancho_mask)
			{
				$this->tablaFinal .= "style=\"width : " . $ancho_mask . "ex\"" ;
			}

			$this->tablaFinal .= ">";
			$this->tablaFinal .= $atributos["DescColumna"];
			}
			else
				$this->tablaFinal .= "\t<td>";
		}
		$this->nodoPadre = "Columna";
		break;
		case "Grupo":
		$this->seccionActual = $nombre;
		$this->tablaFinal .= "<tr class=\"titGrupo\">\n\t<td colspan=\"{$this->numColumnas}\">".
						"Tipo: {$atributos["TiGrupo"]} - $nombre:{$atributos["DescGrupo"]}</td>\n</tr>\n";
		break;
		case "Fila":
		$this->numFilaActual++;
		$estiloFila = ($this->numFilaActual%2 == 0)  ? "filaGrupoPar":"filaGrupoImpar";
		$this->tablaFinal .= "<tr class=\"$estiloFila\">\n";
		break;
/*        case "InicioOpcion":
		$this->seccionActual = $nombre;
		break;*/
		case "InicioGrupo":
		case "CorteGrupo":
		$this->seccionActual = $nombre;
		$this->numColActual=0;
		break;
		case "CorteReporte":
		$this->seccionActual = $nombre;
		$this->tablaFinal .= "<tr class=\"titCorteReporte\">\n\t<td colspan=\"{$this->numColumnas}\">".
						"TOTAL VALORES</td>\n</tr>\n";
		$this->numColActual=0;
		break;
		case "TipoValor":
		if(!empty( $atributos["Ancho"] ))
		{
			$this->nodoPadre = "Tipo";
			$this->tablaFinal .= "\t<td colspan=\""  . $atributos["Ancho"] .  "\">";
			$this->numColActual = $this->numColActual + $atributos["Ancho"] - 1 ;
		}
		else
		{
			$this->nodoPadre = "Tipo";
			$this->tablaFinal .= "\t<td>";
		}
		break;
		case "ValorColumna":
		$this->nodoPadre = "Valor";
		$this->tablaFinal .= "\t<td>";
		break;
		case "PieReporte":
		break;
	}
	}


	/**
	* Funcion llamada por el parser SAX cada vez que se encuentra un tag de cierre, segun el nombre del
	* tag y sus atributos arma el codigo HTML de la tabla
	* @param Parser Parser SAX con la estructura del archivo
	* @param string <var>nombre</var> nombre del nodo actual
	*/
	function manejarTagCierre($parser, $nombre)
	{

	$this->nodoPadre = "";

	switch($nombre)
	{
		case "Reporte":
		$this->tablaFinal .= "\t</td>\n</tr>\n</table>\n";  //Cierra la tabla contenedora del rep
		$this->seccionActual = "";
		break;
		case "Cabecera":
		$this->tablaFinal .= "</table>\n\t</td>\n</tr>\n";  //Cierra la tabla CABECERA
		$this->seccionActual = "";
		break;
		case "Seleccion":
		$this->tablaFinal .= "</td>\n</tr>\n";
		break;
		case "ValorSeleccion":
		$this->tablaFinal .= "&nbsp; - &nbsp;";
		break;
		case "Columna":
		if($this->seccionActual == "CorteGrupo" || $this->seccionActual == "CorteReporte" ||
			$this->seccionActual == "InicioGrupo" || $this->seccionActual == "InicioOpcion")
				$this->tablaFinal .= "</tr>\n";
		else
				$this->tablaFinal .= "</td>\n";
		break;
		case "Encolumnado":
		$this->tablaFinal .= "</tr>\n";
//        case "InicioOpcion":
		case "InicioGrupo":
		case "CorteGrupo":
		case "CorteReporte":
		$this->seccionActual = "";
		break;
		case "Fila":
		$this->tablaFinal .= "</tr>\n";
		break;
		case "Grupo":
		$this->seccionActual = "";
		break;
		case "PieReporte":
		$this->tablaFinal .= "</table>\n" . "Numero de columnas: " . $this->numColumnas; //Cierra la tabla de GRUPOS/CORTES
		break;
		case "TipoValor":
		$this->nodoPadre = "";
		$this->tablaFinal .= "</td>\n";
		for($i=0; $i<$this->posValorCorte-2; $i++) {
			$this->tablaFinal .= "\t<td x=\"ss\">&nbsp;</td>\n";
		}
		break;
		case "ValorColumna":
		$this->nodoPadre = "";
		$this->tablaFinal .= "</td>\n";
		for($i=$this->posValorCorte; $i<( $this->numColumnas - $this->nodoPadre.$atributos["Ancho"]); $i++) {
			$this->tablaFinal .= "\t<td y=\"tt\">&nbsp;</td>\n";
		}
		break;
		case "ValorEstado":
		break;
	}
	}


	/**
	* Funcion llamada por el parser SAX cada vez que se encuentra un tag de texto, segun el nombre del
	* tag y sus atributos arma el codigo HTML de la tabla
	* @param Parser Parser SAX con la estructura del archivo
	* @param string <var>datos</var> texto hijo del nodo actual
	*/
	function manejarTagTexto($parser, $datos)
	{
	$datosSinBlancos = trim($datos);

/*      $datosEscapados = escapeshellcmd($datosSinBlancos);*/

	if(!empty($datosSinBlancos))
		$this->tablaFinal .= $datosSinBlancos  ;
	/*
	if(!empty($datosEscapados))
		$this->tablaFinal .= $datosEscapados  ;
*/
	}

}
?>